#!/usr/local/bin/ruby
#Program to construct HTML out of a collection of RFC's
#
#Copyright (C) 2002  Peter Minten  <silvernerd@users.sf.net>
#
#Licensed under the GNU General Public License v2 
#or at your option any later version.

#Requires REXML (http://www.germane-software.com/software/rexml)

#Usage: ruby rfc2html.rb <list of all the .rfc files, wildcards allowed>

#Example: ruby rfc2html.rb *.rfc

#Helper functions
#Why bother writing 'write' if you can shorten it to 'w'
class IO
  def w(str)
    self.write(str)
  end
end

def integer? (str)
  str.each_byte{|byte|
    if byte < 48 or byte > 57 then
      return false
    end
  }
  return true
end

def datevalid? (str)
  strs = str.to_s.split('-')
  return false unless integer?(strs[0])
  return false unless strs[0].length == 2
  return false unless strs[0].to_i > 0 and strs[0].to_i < 32
  return false unless integer?(strs[2])
  return false unless strs[2].length == 4
  return false unless strs[2].to_i > 2001
  unless strs[1] =~ /jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec/ then
    p "ERROR POINT"
    return false
  end
  return true
end

class String
  def startcase
    str = self.downcase
    str[0] -= 32
    str.to_s
  end
end

#Program starts here

require 'rexml/document'
include REXML #avoid having to type REXML:: all the time

strings = readlines

string = ""

strings.each{|line|
string = string + line
}

string = "<root>" + string + "</root>"

#Replace <fulltext> with <fulltext><![CDATA[ to avoid parsing the text
string.gsub!("<fulltext>", "<fulltext><![CDATA[")
#And the other end too
string.gsub!("</fulltext>", "]]></fulltext>")

doc = Document.new (string)

#The global variables
$rfcnumbers = Array.new
$rfcnames = Array.new
$rfcmaintainers = Array.new
$rfcedates = Array.new
$rfcddates = Array.new
$rfcstatuses = Array.new
$rfcareas = Array.new
$rfcversions = Array.new
$rfcdescriptions = Array.new
$rfcfulltexts = Array.new
$rfcnumbertaken = Array.new

$i = 0

#ITERATION FAILS
#For every RFC element do this
doc.elements.each ("root/rfc") {|el|
  #Parse the child elements
  el.elements.each{|e|
    case e.name
      when "number"
      $rfcnumbers[$i] = e.text
      when "name"
      $rfcnames[$i] = e.text
      when "maintainers"
      $rfcmaintainers[$i] = e.text
      when "dateofentrance"
      $rfcedates[$i] = e.text
      when "dateofdecision"
      $rfcddates[$i] = e.text
      when "area"
      $rfcareas[$i] = e.text
      when "status"
      $rfcstatuses[$i] = e.text
      when "version"
      $rfcversions[$i] = e.text
      when "description"
      $rfcdescriptions[$i] = e.text
      when "fulltext"
      $rfcfulltexts[$i] = e.text
    end
  }
  $i += 1
}

#We now have our information, let's go on by checking for validity stuff
for a in 0..$i-1 do
  #It's very usefull to shorten $rfcnumbers[a] to n
  n = $rfcnumbers[a]

  #Check if there is an RFC number
  raise ("RFC number missing") if n.nil?
  
  #Check for integer number, all RFC's must have good numbers 
  raise ("Invalid RFC number: " + n) unless integer?(n)

  #Check for double RFC numbers
  if $rfcnumbertaken[n.to_i] then
    raise ("RFC number already used: " + n)
  else
    $rfcnumbertaken[n.to_i] = true
  end

  #Check if a name is given
  raise ("No name given in RFC: " + n) if $rfcnames[a].nil?

  #Check if maintainers are given
  raise ("No maintainers given in RFC: " + n) if $rfcmaintainers[a].nil?  

  #Check if status is given
  raise ("No status given in RFC: " + n) if $rfcstatuses[a].nil?  

  #Check if status is valid
  if $rfcstatuses[a].downcase !~ /rejected|being considered|accepted/ then
    raise ("Invalid status given in RFC: " + n + " status: " + $rfcstatuses[a])
  end

  #Check if date of entrance is given
  raise ("No date of entrance given in RFC: " + n) if $rfcedates[a].nil?

  #Check if date of entrance is valid 
  unless datevalid?($rfcedates[a])
    raise ("Invalid date of entrance in RFC: " + n + " date: " + $rfcedates[a])
  end

  #Set date of decision to '-' under certain circumstances
  $rfcddates = "-" if $rfcddates[a].strip == '-'
  $rfcddates = "-" if $rfcddates[a].nil?
  $rfcddates = $rfcddates.to_s

  #Check if date of decision is valid
  unless $rfcddates == "-" 
    unless datevalid?($rfcddates[a])
      raise ("Invalid date of decision in RFC: " + n + " date: " + $rfcddates[a])
    end
  end

  #Check if area is given
  raise ("No area given in RFC: " + n) if $rfcareas[a].nil?

  #Check if version is given
  raise ("No version given in RFC: " + n) if $rfcversions[a].nil?

  #Check if version is valid
  unless integer?($rfcversions[a]) 
    raise ("Invalid version number in RFC: " + n + " number: " + $rfcversions[a])
  else
    if $rfc_versions.to_i < 0 #It should be < 1 but Ruby doesn't accept that
     raise ("Invalid version number in RFC: " + n + " number: " + $rfcversions[a])
    end
  end
  
  #Check if description is given
  raise ("No description given in RFC: " + n) if $rfcdescriptions[a].nil?

  #Check if full text is given
  raise ("No full text given in RFC: " + n) if $rfcfulltexts[a].nil?
end

#Check done, now generate the full text html files
#The files are put into a subdir called rfc_html
system("mkdir rfc_html") unless test ("d", "rfc_html")

for a in 0..$i-1 do
  filename = "rfc_html/rfc_" + $rfcnumbers[a] + ".html"
  open (filename, "w") {|f|
    f.w ("<html><head><title>RFC " + $rfcnumbers[a] + ": " + $rfcnames[a] \
	 + "</title></head><body>")

    case $rfcstatuses[a].downcase
      when "accepted"
      color = "green"
      when "being considered"
      color = "yellow"
      when "rejected"
      color = "red"
    end
    f.w ("<h1>RFC " + $rfcnumbers[a] + ": " + $rfcnames[a] + "</h1>")
    f.w ("<tr><td bgcolor = \"" + color + "\"><h2>")
    f.w ("<b>Maintainers: </b>" + $rfcmaintainers[a] + "<br>")
    f.w ("<b>Date of entrance: </b>" + $rfcedates[a] + "<br>")
    f.w ("<b>Date of decision: </b>" + $rfcddates[a].to_s + "<br>")
    f.w ("<b>Area: </b>" + $rfcareas[a] + "<br>")
    f.w ("<b>Status: </b>" + $rfcstatuses[a].startcase + "<br>")
    f.w ("<b>Version: </b>" + $rfcversions[a] + "<br>")
    f.w ("<b>Description: </b>" + $rfcdescriptions[a] + "<br>")
    f.w ("<br><hr><br></h2>")
    f.w ($rfcfulltexts[a])
    f.w ("<br><hr><div align = \"center\"><h2><a href=\"summary.html\">" + \
	 "Back to RFC summary page<a><h2></div>")
    f.w ("</body></html>")
  }
end

#Now generate the summary page
open ("rfc_html/summary.html", "w") {|f|
  f.w ("<html><head><title>DotGNU RFC summary page</title></head>")
  f.w ("<body><table width=\"100%\"")
  for a in 0..$i-1 do
    case $rfcstatuses[a].downcase
      when "accepted"
      color = "green"
      when "being considered"
      color = "yellow"
      when "rejected"
      color = "red"
    end
    f.w ("<tr><td bgcolor = \"" + color + "\"><h3>")
    f.w ("<b>RFC " + $rfcnumbers[a] + "</b><br>")
    f.w ("<b>Name: </b>" + $rfcnames[a] + "<br>")
    f.w ("<b>Maintainers: </b>" + $rfcmaintainers[a] + "<br>")
    f.w ("<b>Date of entrance: </b>" + $rfcedates[a] + "<br>")
    f.w ("<b>Date of decision: </b>" + $rfcddates[a].to_s + "<br>")
    f.w ("<b>Area: </b>" + $rfcareas[a] + "<br>")
    f.w ("<b>Status: </b>" + $rfcstatuses[a].startcase + "<br>")
    f.w ("<b>Version: </b>" + $rfcversions[a] + "<br>")
    f.w ("<b>Description: </b>" + $rfcdescriptions[a] + "<br>")
    f.w ("<div align = \"center\"><b><a href = \"" + \
	 "rfc_" + $rfcnumbers[a] + ".html" + \
	 "\">Link to RFC page</b></a></div><br>")    
    f.w ("</h3></td></tr>")
  end
}
